//+------------------------------------------------------------------+
//|                                           OnChart Stochastic.mq4 |
//|                                                           mladen |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 DarkSlateGray
#property indicator_color2 DarkSlateGray
#property indicator_color3 DarkSlateGray
#property indicator_color4 LimeGreen
#property indicator_color5 PaleVioletRed
#property indicator_color6 PaleVioletRed
#property indicator_color7 Red
#property indicator_width4 2
#property indicator_width5 2
#property indicator_width6 2
#property indicator_style1 STYLE_DOT
#property indicator_style7 STYLE_DOT

//---- parameters
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    KPeriod         = 14;
extern int    Slowing         = 3;
extern int    DPeriod         = 3;
extern int    maPeriod        = 20;
extern int    maMethod        = 1;
extern int    maPrice         = 0;
extern int    overBought      = 80;
extern int    overSold        = 20;
extern bool   ColorOnSlope    = false;
extern bool   Interpolate     = true;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = true;
extern bool   alertsEmail     = true;
extern string soundFile       = "alert2.wav"; 

//---- buffers
//
//
//
//

double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];
double Buffer5[];
double stoDa[];
double stoDb[];
double state[];
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;
int    atrTimeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexBuffer(2,Buffer3);
   SetIndexBuffer(3,Buffer4);
   SetIndexBuffer(4,stoDa);
   SetIndexBuffer(5,stoDb);
   SetIndexBuffer(6,Buffer5);
   SetIndexBuffer(7,state);

   //
   //
   //
   //
   //
   
   timeFrame = stringToTimeFrame(TimeFrame);
      switch (timeFrame)
         {
            case PERIOD_M1:  atrTimeFrame = PERIOD_M30; break;
            case PERIOD_M5:  atrTimeFrame = PERIOD_H1;  break;
            case PERIOD_M15: atrTimeFrame = PERIOD_H4;  break;
            case PERIOD_M30: atrTimeFrame = PERIOD_D1;  break;
            case PERIOD_H1:  atrTimeFrame = PERIOD_D1;  break;
            case PERIOD_H4:  atrTimeFrame = PERIOD_W1;  break;
            case PERIOD_D1:  atrTimeFrame = PERIOD_W1;  break;
            default:         atrTimeFrame = PERIOD_MN1;
         }
      indicatorFileName = WindowExpertName();
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
      
      //
      //
      //
      //
      //
               
   IndicatorShortName(timeFrameToString(timeFrame)+" Stochastic");
   return(0);
}
int deinit() { return(0); }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit=MathMax(Bars-counted_bars,MathMin(5*atrTimeFrame/Period(),Bars-1)); 
         if (returnBars) { Buffer1[0] = MathMin(limit+1,Bars-1); return(0); }  
      

   //----
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      if (state[limit] ==-1) CleanPoint(limit,stoDa,stoDb);
      for (int i=limit; i>=0; i--)
      {
         int r = iBarShift(NULL,atrTimeFrame,Time[i]);
            double avgRange     = iATR(NULL,atrTimeFrame,maPeriod,r);
            double maValue      = iMA(NULL,0,maPeriod,0,maMethod,maPrice,i);
            double stochValue   = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,maMethod,0,MODE_MAIN,  i);
            double stochValue2  = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,maMethod,0,MODE_MAIN,  i+1);
            double signalValue  = iStochastic(NULL,0,KPeriod,DPeriod,Slowing,maMethod,0,MODE_SIGNAL,i);
  
               //
               //
               //
               //
               //
               
               stoDa[i]   = EMPTY_VALUE;
               stoDb[i]   = EMPTY_VALUE;
               state[i]   = state[i+1];
               Buffer1[i] = maValue;
               Buffer2[i] = maValue+(avgRange*(overBought-50)/100);
               Buffer3[i] = maValue-(avgRange*(50-  overSold)/100);
               Buffer4[i] = maValue+(stochValue -50)/100*avgRange;
               Buffer5[i] = maValue+(signalValue-50)/100*avgRange;
               if (ColorOnSlope)
               {
                  if (Buffer4[i] > Buffer4[i+1]) state[i] =  1;
                  if (Buffer4[i] < Buffer4[i+1]) state[i] = -1;
               }
               else
               {
                  if (Buffer4[i] > Buffer5[i])   state[i] =  1;
                  if (Buffer4[i] < Buffer5[i])   state[i] = -1;
               }            
               if (state[i] ==-1) PlotPoint(i,stoDa,stoDb,Buffer4);
         }
   manageAlerts();      
   return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (state[limit] ==-1) CleanPoint(limit,stoDa,stoDb);
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Buffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,0,y);
         Buffer2[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,1,y);
         Buffer3[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,2,y);
         Buffer4[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,3,y); 
         Buffer5[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,6,y); 
         state[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",KPeriod,Slowing,DPeriod,maPeriod,maMethod,maPrice,overBought,overSold,ColorOnSlope,7,y); 
         stoDa[i]   = EMPTY_VALUE;
         stoDb[i]   = EMPTY_VALUE;
           
         //
         //
         //
         //
         //
               
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //
 
         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               Buffer1[i+k] = Buffer1[i] + (Buffer1[i+n] - Buffer1[i]) * k/n;
               Buffer2[i+k] = Buffer2[i] + (Buffer2[i+n] - Buffer2[i]) * k/n;
               Buffer3[i+k] = Buffer3[i] + (Buffer3[i+n] - Buffer3[i]) * k/n;
               Buffer4[i+k] = Buffer4[i] + (Buffer4[i+n] - Buffer4[i]) * k/n; 
               Buffer5[i+k] = Buffer5[i] + (Buffer5[i+n] - Buffer5[i]) * k/n;             
           }          
    }
    for (i=limit;i>=0;i--) if (state[i] ==-1) PlotPoint(i,stoDa,stoDb,Buffer4); 
    manageAlerts();
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
return(s);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (state[whichBar] != state[whichBar+1])
      {
         if (state[whichBar] == 1)  doAlert(whichBar,"up");
         if (state[whichBar] ==-1)  doAlert(whichBar,"down");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

           message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," OnChart Stochastic ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " OnChart Stochastic "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}
